package j2ee.permanent.storage;

import java.io.FileReader;
import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.FileInputStream;

import org.apache.commons.httpclient.HttpException;
import org.apache.webdav.lib.WebdavResource;
import org.apache.commons.httpclient.HttpsURL;
import org.apache.commons.httpclient.URIException;
import java.net.MalformedURLException;

import java.util.ResourceBundle;
import java.util.MissingResourceException;

/**
 * <p>This class provides methods to connect to a given location (provided by a .properties-file)
 * at the DFS file system at CERN through applications deployed to the J2EE Public Service 
 * as a WAR-file<p> 
 *
 * <p>It works under the assumption that a dfs.properties file with correct location/login 
 * information has been placed by the application owner in /WEB-INF/classes/ directory in the
 * WAR-file. The class can also be used without having a property file in the WAR-file, and instead
 * using the methods below that provides the location/login information directly through
 * the input parameters of the save/retrieve-methods</p>
 *
 * <p>How to use this class:</p>
 *
 * <p>To retrieve or save a file using the save(String location, String username, String password) or the
 * retrieve(String location, String username, String password) you only have to specify the correct input parameters
 * and it should work.</p>
 *
 * <p>To use the save() or the retrieve() method, you have to place a dfs.properties file inside the /WEB-INF/classes
 * directory of your WAR-file. The dfs.properties file has to have the following keys</p>
 *
 * <ul>
 *	<li>dfs.location, e.g. /dfs/Users/first-letter-of-username/username/directory-of-your-choice</li>
 *	<li>dfs.username</li>
 * 	<li>dfs.password</li>
 * </ul>
 *
 * <p>It is very important that the key dfs.location in the dfs.properties file is correct, otherwise it will not work</p>
 *
 * @author Ola Holmestad
 * @version 1.0 2006-09-12
 * @see java.io.File
 *
 */

public class File extends java.io.File {

	//variables that hold key information about where to connect
	private String location = "";
	private String username = "";
	private String password = "";
	private	String fileName = "";

	//the webdavresource that is used to connect to DFS
	WebdavResource wdr = null;

	/**
   	 * <p>Standard constructor for the class. Creates an instance of the java.io.File class</p>
 	 *
	 * @param fileName The name of the file
	 * 
	 */

	public File(String fileName) {

		super(System.getProperty("java.io.tmpdir")+"/"+fileName);

		this.fileName = fileName;

	}

	/**
	 * <p>Method that saves the given file and its content to the DFS system. </p>
	 * 
 	 * <p>This method checks whether it can find the dfs.properties file and performs the 
	 * save (through the saveFile()-method) if it can, otherwise it returns false or 
	 * an exception describing what went wrong</p>
	 *
	 * @return true if file is successfully saved, false otherwise
	 * @throws IOException if something goes wrong during IO
	 * @throws MissingResourceException if the dfs.properties can't be found or have bad/missing information
	 * 
	 */

	public boolean save() throws IOException, MissingResourceException {

		if (getLocationInfo()) {

			if (saveFile()) {

				return true;

			}

			else {

				return false;

			}

		}

		else {

			return false;

		}


	}

	 /**
         * <p>This method can be used if the user does not want to provide a dfs.properties file.</p>
	 * 
	 * <p>Instead, the location/login information can be given as parameters to this method and 
	 * it will directly try to save the file</p>
  	 *
	 * @param location the location at dfs, of the form /dfs/Users/first-letter-of-username/username/directory-of-choice,
	 * 		e.g. /dfs/Users/t/testuser/J2EEPS/
	 * @param username The user's username
	 * @param password The user's password
	 * @return true if file is successfully saved, false or an IOexception otherwise
         * @throws IOException if something goes wrong during the saving of the file to the DFS Webdav server
	 *
	 */

        public boolean save(String location, String username, String password) throws IOException{

                this.location = location;
                this.username = username;
                this.password = password;

                if (saveFile())

                        return true;

                else

                        return false;

        }

	/**
	 * <p>This method retrieves a file from the DFS file system location and places it in the 
	 * default temporary directory at the J2EEPS container</p>
	 *
	 * <p>If it can find the dfs.properties file, then it tries to connect and retrieve the specified
	 * file from the DFS location given in dfs.properties</p>
	 *
	 * @return true if the file could be retrieved
	 * @throws IOException if something goes wrong during retrieval
	 * @throws MissinResourceException if the dfs.properties file can't be found
	 * 
	 */
 
	public boolean retrieve() throws IOException, MissingResourceException {

		if (getLocationInfo()) {

			if (retrieveFile()) {

				return true;

			}

			else {

				return false;

			}

                }

                else {

                        return false;

                }

	}

	/**
	 * <p>This method is an alternative method where the user don't have to include a dfs.properties-file
	 * in his/her's WAR-file.</p>
	 *
	 * <p>Instead, by giving the location/login information through the input parameters, this method will
	 * try to retrieve the file from DFS</p>
	 *
	 * @param location the location at dfs, of the form /dfs/Users/first-letter-of-username/username/directory-of-choice,
         *              e.g. /dfs/Users/t/testuser/J2EEPS/
	 * @param username the user's username
	 * @param password the user's password
 	 * @return true if file is successfully retrieved
	 * @throws IOException if something goes wrong during retrieval
	 *
	 */

	public boolean retrieve(String location, String username, String password) throws IOException {

                this.location = location;
                this.username = username;
                this.password = password;

                if (retrieveFile()) {

                        return true;

                }

                else {

                        return false;

                }

        }

	/**
	 * <p>This method retrieves the dfs.properties file and populates the member variables
	 * that provide the location/login information for the DFS Webdav Connection</p>
	 *
	 * @return true if everything goes ok, throws an MissinResourceException otherwise
	 * @throws MissingResourceException if dfs.properties can't be found in the container
	 *
	 */

	private boolean getLocationInfo() throws MissingResourceException {
		
                ResourceBundle bundle = ResourceBundle.getBundle("dfs");			

		location = parseLocation(bundle.getString("dfs.location"));
		username = bundle.getString("dfs.username");
		password = bundle.getString("dfs.password");				

		return true;
		
	}

	/**
	 * <p>This method creates an HttpsURL to the location at DFS and connects a WebdavResource to this url.</p>
	 *
	 * <p>It then executes the getMethod() to put the file in the default temporary directory for the container</p>
	 * 
	 * @return true if everyting goes okay, IOException otherwise
	 * @throws IOException if an IO error occurs
	 *
	 */

	private boolean retrieveFile() throws IOException {

		 try {

	                //creating an HttpsURL to the DFS file system at CERN
                        HttpsURL hrl = new HttpsURL("https://dfs.cern.ch/"+location+"/"+fileName);
                        hrl.setUserinfo(username, password);
                        wdr = new WebdavResource(hrl);

			if (wdr != null) {
		
				if (wdr.getMethod(this)) {

					return true;

				}

				else {
	
					throw new IOException("Was not able to retrieve file from "+wdr.getPath());
		
				}

			}

			else {

				throw new IOException("Can not connect to the DFS file system. Check your path in the dfs.properties file and make sure it's correct");

			}

                }

                catch(MalformedURLException mue) {

			throw new IOException("MalformedURLException occurred: "+mue.getMessage());

                }

                catch(HttpException he) {

			throw new IOException("Not valid location at DFS (file might not exist or wrong path) or wrong username/password. Check your dfs.properties and make sure that the file exists");

                }

	}

	/**
	 * <p>This method creates a HttpsURL to the DFS lcoation and then tries to establish a WebdavResource 
	 * by using the url.</p>
	 * 
	 * <p>If this is successfull, the putMethod()- are used to save the file to the WebdavResource
	 * at DFS</p>
	 *
	 * @return true if the file is saved, exception throwed otherwise
	 * @throws IOException if an IO error occurs
	 *
	 */

	private boolean saveFile() throws IOException {	

		try {
		
			HttpsURL hrl = new HttpsURL("https://dfs.cern.ch/"+location);
        	        hrl.setUserinfo(username, password);
                        wdr = new WebdavResource(hrl);

			//to make sure that the filename doesn't have '/' in it
			String realName = parse();

			if (wdr != null) {

				if (wdr.putMethod(wdr.getPath()+"/"+realName, this))
				
					return true;

				else 

					throw new IOException("File "+wdr.getPath()+"/"+fileName+" could not be saved");

			}

			else {

				throw new IOException("Was not able to connect to the DFS WebDAV server");

			}
	
		}

		catch (HttpException httpe) {
		
			throw new IOException("HttpException occurred: "+httpe.getMessage());
	
	
		}

		catch (URIException urie) {

			throw new IOException("URIException occurred: "+urie.getMessage());

		}
			
	} 

	/**
	 * <p>This methods removes (if present) a '/' from the end of the filename. It also
	 * takes away the directory information from the filename (e.g. /home/user/example.file/ will 
	 * get parsed to only example.file)</p>
	 *
	 * @return the parsed string
	 *
	 */

	private String parse() {

		if (fileName.endsWith("/")) 

			fileName = fileName.substring(0, fileName.length());

		int start = fileName.lastIndexOf("/");
		
		if (start != -1)		

			return fileName.substring(start+1);

		else

			return fileName;

	}

	/**
	 * <p>This method checks if a '/' is present at the start and at the end of the location 
	 * given in the dfs.properties or through the method call, and removes them if they are there</p>
	 *
	 * @param s The location that should be parsed
	 * @return the parsed location without '/' at the start and end
	 *
	 */

	private String parseLocation(String s) {

		if (s.startsWith("/")) {

			s = s.substring(1);

		}

		if (s.endsWith("/")) {

			int length = s.length();
			s = s.substring(0, s.length()-1);

		}

		return s;

	}

}
	
