package j2ee.io;

import java.net.URI;
import java.net.MalformedURLException;
import java.net.URL;

import java.util.ResourceBundle;
import java.util.MissingResourceException;
import java.util.ArrayList;

import org.apache.commons.httpclient.HttpsURL;
import org.apache.commons.httpclient.HttpException;
import org.apache.commons.httpclient.URIException;

import org.apache.webdav.lib.WebdavResource;

import java.io.IOException;
import java.io.FilenameFilter;
import java.io.FileFilter;

/**
 * <p>This version of the File class that inherits from the java.io.File class is designed specially
 * to let user's at the J2EEPS at CERN use and write to files stored at their local user area at
 * DFS.</p> 
 *
 * <p>For this to work, the user has to have a file named dfs.properties inside his/hers war-file, 
 * where the following keys has to be present</p>
 * 
 * <ul><li>dfs.location</li><li>dfs.username</li><li>dfs.password</li></ul>
 *  
 * <p>If this information is present and valid, each time the user creates an object of this class, it will
 * be searched for the same file at the DFS location, and the user can call all the methods available
 * in this class and will manipulate the file at DFS. </p>
 * 
 * @author Ola Holmestad
 * 
 */

public class File extends java.io.File {

	private String location = "";
	private String username = "";
	private String password = "";

	private WebdavResource wdr = null;

	private HttpsURL hrl = null;

	private String path = null;

	/**
	 * <p>A constructor that creates the file/directory specified in the temporary directory 
	 * locally at the J2EEPS container in question. </p>
	 *
	 * <p>If it can find information about user credentials for DFS, it tries to retrieve the file
	 * and put it in the temporary directory for it to be further accessed</p>
	 *
	 * <p>If the input is a directory (e.g. ends with /) it will create the directory at DFS if 
	 * user credentials is available</p>
	 *
	 * <p>The method uses the constructor of the superclass and adds the temporary directory path
	 * directly when calling the superclass constructor</p>
	 *
	 * @param pathName The path for the file/directory
	 *
	 */

	public File(String pathName) {

		//creating the file in the default temporary directory with the specified pathname
		super(System.getProperty("java.io.tmpdir")+"/"+pathName);
		path = System.getProperty("java.io.tmpdir")+"/"+pathName;

		String temp = path.replaceFirst(System.getProperty("java.io.tmpdir")+"/", "");

		if (getLocationInfo() && !(temp.equals("."))) {

			//checking if it is a directory
			if (path.endsWith(separator)) 

				getDirectory(vectorize(path));

			else 

				getFile(vectorize(path));			

		}

	}

	/**
         * <p>A constructor that creates the file/directory specified with parent and child string 
	 * in the temporary directory locally at the J2EEPS container in question.</p>
         *
         * <p>If it can find information about user credentials for DFS, it tries to retrieve the file
         * and put it in the temporary directory for it to be further accessed</p>
         *
         * <p>If the input is a directory (e.g. ends with /) it will create the directory at DFS if   
         * user credentials is available</p>
         *
	 * <p>The method uses the constructor of the superclass and adds the temporary directory path
         * directly when calling the superclass constructor</p>
         *
         * @param parent The path for the parent of the file
	 * @param child The name of the child file
         *
         */

	public File(String parent, String child) {

		//creating the file in the default temporary directory with the specified pathname
		super(System.getProperty("java.io.tmpdir")+"/"+parent, child);
		path = System.getProperty("java.io.tmpdir")+"/"+parent+separator+child;

		if (getLocationInfo()) {	

			//checking if it is a directory	
			if (path.endsWith(separator))

				getDirectory(vectorize(path));

			else

				getFile(vectorize(path));
         
                }

	}

	/**
         * <p>A constructor that creates the file/directory specified with parent and child string
         * in the temporary directory locally at the J2EEPS container in question.</p>
         *
         * <p>If it can find information about user credentials for DFS, it tries to retrieve the file
         * and put it in the temporary directory for it to be further accessed</p>
         *
         * <p>If the input is a directory (e.g. ends with /) it will create the directory at DFS if
         * user credentials is available</p>
         *
	 * <p>The method uses the constructor of the superclass and adds the temporary directory path
         * directly when calling the superclass constructor</p>
         *
         * @param parent The parent file object
         * @param child The name of the child file   
         *
         */

	public File(File parent, String child) {

		super(System.getProperty("java.io.tmpdir")+"/"+parent.getAbsolutePath(), child);
		path = System.getProperty("java.io.tmpdir")+"/"+parent.getAbsolutePath()+separator+child;

		if (getLocationInfo()) {

			//checking if it is a directory
			if (path.endsWith(separator))

				getDirectory(vectorize(path));

			else

				getFile(vectorize(path));

		}
         
	}

	/**
	 * <p>This method creates the directory with all parents specified in the input array, both locally and at DFS</p>
	 *
	 * <p>It uses the method File.createDirectory() to create the directories at DFS. Is only used if a valid 
	 * connection to DFS can be established</p>
	 *
	 * @param paths - The String array with the directory itself and all parents that should be created
	 *
	 */
           
	private void getDirectory(String[] paths) {

		String pathString = "";
 	
		//creating the directories at DFS one by one	
		for (int i = 0; i < paths.length; i++) {

			//making sure that the last directory is interpreted correctly (by adding a "/")
			if (i == (paths.length-1)) {

				pathString += paths[i];

				//create the directory locally
				java.io.File dir = new java.io.File(System.getProperty("java.io.tmpdir")+"/"+pathString+"/");
				dir.mkdir();

			}

			//making sure that no duplicate "/"'s are present
			else { 

				pathString += paths[i]+separator;

				//create the directory locally
				java.io.File dir = new java.io.File(System.getProperty("java.io.tmpdir")+"/"+pathString);
				dir.mkdir();

			}

			//creates the directory
			createDirectory(pathString);

		}	

	}

	/**
	 * <p>Not implmenented, always returns null</p>
	 *
	 * @return null
	 *
	 */

	public java.net.URL toURL() {

		return null;

	}

	/**
	 * <p>Not implemented, always returns null</p>
	 *
	 * @return null
	 *
	 */

	public java.net.URI toURI() {

		return null;

	}

	/**
	 * <p>This method takes the file from the DFS location and copies it to the temporary directory 
	 * locally at the J2EEPS server, for there to be further accessed/written to from the user's application</p>
	 *
	 * @param paths - The String array with all the path information for the parents of the file and the file itself
	 *
	 */

	private void getFile(String[] paths) {

		String fileLocation = "";

		//constructing the file path from the input string array
		for (int i = 0; i < paths.length; i++) {

			fileLocation += separator+paths[i];

		}

		//removing the temporary directory part of the pathname
		fileLocation = fileLocation.replaceFirst(System.getProperty("java.io.tmpdir"), "");

		try {
 
			//try to connect to the specific location at DFS               
 			if (setDFSConnection("https://dfs.cern.ch/"+location+fileLocation)) {                        

				wdr = new WebdavResource(hrl);		

				//the file exists at DFS, now we need to save it in the temporary directory at the local server

				if (wdr != null) {

					if (wdr.isCollection()) 
	
						return;

					//creating any missing directories locally          	 
					java.io.File parent = getParentFile();
						
                                        if (parent != null) {
                 
                                                if (!parent.mkdir())
                
                                                        parent.mkdirs();
         
                                        }       

					//creating a dummy file to the correct location where the file from DFS will be put
					java.io.File f = new java.io.File(System.getProperty("java.io.tmpdir")+"/"+trimFirst(fileLocation));

                        	        wdr.getMethod(f);
    	         	                                   
                        	}
         	
                	                        	
                     	}

			else {

				return;
			
			}	

		}

		catch(MalformedURLException mue) {  
                                           
			return;

                }
         
                catch(HttpException he) {			                

			//this means that no file exists at DFS, creating the parent directories locally
			
			if (isDirectory())

				return;
                 
                 		java.io.File f = new java.io.File(System.getProperty("java.io.tmpdir")+"/"+getAbsolutePath());                        
                        java.io.File parent = getParentFile();             
		
                        if (parent != null && !(parent.getAbsolutePath()).equals(System.getProperty("java.io.tmpdir"))) {

			        		if (!parent.mkdir()) {                            
	
                                        boolean value = parent.mkdirs();

				}                         

	               	}

                }

		catch (URIException urie) {

			return;

		}

		catch (IOException ioe) {

			return;

		}

	}

	/** 
	 * <p>Helper method that ensures that the input does not start or end with a "/". Is used
	 * to ensure conformity when treating location strings in the class</p>
	 *
	 * @param s - The string to be checked for starting or ending "/"'s
	 * @return - The parsed string
	 *
	 */

	private String parseLocation(String s) {
                
                if (s.startsWith(separator)) {

                        s = s.substring(1);

                }

                if (s.endsWith(separator)) {
         
                        int length = s.length();
                        s = s.substring(0, s.length()-1);
          
                }
         
                return s;
           
        }

	/**
	 * <p>Checks if it can find a dfs.properties file in the user's directory. </p>
	 *
	 * @return true if a connection is possible to DFS, false otherwise
	 *
	 */

	private boolean getLocationInfo() {

		try {

			ResourceBundle bundle = ResourceBundle.getBundle("dfs");
         
        	        location = parseLocation(bundle.getString("dfs.location"));
               		username = bundle.getString("dfs.username");
	                password = bundle.getString("dfs.password");

			return true;
		
		}

		catch(MissingResourceException mre) {

			return false;

		}        
                
	}

	/**
	 * <p>Helper method that splits the input around the separator defined (usually "/")</p>
	 *
	 * <p>Is used to give an array of parent directories and the directory/file itself</p>
	 *
	 * @param input - The string to be split
	 * @return A String array with the parent directories and the directory/file itself
	 *
	 */
	
	private String[] vectorize(String input) {

			//remove the temporary directory part of the file path
			input = input.replaceFirst(System.getProperty("java.io.tmpdir"), "");			
			input = trim(input);

			return input.split(separator);

	}

	/**
	 * <p>Deletes the file or directory denoted by this abstract pathname. Should be used with care</p>
	 *
	 * @return true if and only if the file or directory is successfully deleted; false otherwise
	 *
	 */

	public boolean delete() {

		String temp = path.replaceFirst(System.getProperty("java.io.tmpdir")+"/", "");

		if (temp.equals("."))

			return false;

		//if access to DFS is ready
		if (getLocationInfo())

			return deleteFile(getAbsolutePath());

		//if there is no acccess information
		else 

			return super.delete();

	}

	/**
	 * <p>Returns the absolute pathname string of this abstract pathname.</p>
	 * 
	 * <p>If a connection to DFS is possible, it will remove the temporary directory of the J2EEPS container
	 * from the path, and just show the DFS part from the user's location directory in dfs.properties. </p>
	 *
	 * <p>Otherwise it will return the superclass method</p>
	 *
	 * @return The absolute pathname string denoting the same file or directory as this abstract pathname
	 *
	 */

	public String getAbsolutePath() {

		if (getLocationInfo()) {

			String temp = path.replaceFirst(System.getProperty("java.io.tmpdir")+"/", "");

			if (temp.equals("."))

				return temp;			

			String s = path;
			//removing the temporary directory part of the path
			s = s.replaceFirst(System.getProperty("java.io.tmpdir")+"/", "");
	
			return s;
				
		}

		else 

			return super.getAbsolutePath();

	}

	/**
	 * <p>This method is not possible to implement correctly,  as there is no indication to DFS about when the Java
	 * virtual machine terminates.</p>
	 *
	 * <p>It will, if called, call the superclass method and the file will be deleted locally on termination as described 
	 * for the superclass constructor (but no deletion at DFS)</p>
	 *
	 */ 

	public void deleteOnExit() {

		super.deleteOnExit();
		
		return;

	}

	/**
	 * <p>This method deletes the file at DFS. It is used by the File.delete() method</p>
	 *
	 * @param path - The path to the file/directory that should be deleted
	 * @return true if the file/directory was deleted, false otherwise
	 *
	 */
	 
	private boolean deleteFile(String path) {

		//if not possible to connect, return false because no deletion was done
		if (!setDFSConnection("https://dfs.cern.ch/"+path))

			return false;

		try {

			wdr = new WebdavResource(hrl);

			if (wdr != null) {

				//deleting the file at DFS	
				return wdr.deleteMethod();

			}			

			else 

				return false;

		}

		catch(HttpException httpe) {

			return false;

		}

		catch (IOException ioe) {

			return false;
	
		}
	
	}

	/**
	 * <p>This method sets the DFS connection according to the specified input. It does not check if this 
	 * is a valid location, that has to be done after by trying to create the WebdavResource connection</p>
	 *
	 * @param location - the DFS location
	 * @return true if the connection is okay, false otherwise
	 *
	 */ 

	private boolean setDFSConnection(String location) {

		try {

			hrl = new HttpsURL(location);
        	        hrl.setUserinfo(username, password);

			return true;

		}

		catch (HttpException httpe) {

			return false;

		}

		catch (URIException urie) {

			return false;

		}

	}

	/**
	 * <p>If a connection to DFS is possible, this will return the same as getAbsolutePath(). If not, 
	 * it returns the result of the superclass method</p>
	 *
	 * @return The canonical pathname string denoting the same file or directory as this abstract pathname
	 *
	 */

	public String getCanonicalPath() throws java.io.IOException {

		if (getLocationInfo())

			return getAbsolutePath();

		else 

			return super.getAbsolutePath();

	}

	/**
	 * <p>Returns the pathname string of this abstract pathname's parent, or null if this pathname does not name a parent directory.</p>
	 *
	 * @return The pathname string of the parent directory named by this abstract pathname, or null if this pathname does not name a parent
	 *
	 */

	public String getParent() {

		String temp = path.replaceFirst(System.getProperty("java.io.tmpdir")+"/", "");

		if (temp.equals("."))

			return null;

		if (getLocationInfo()) {

			java.io.File f = getParentFile();
		
			if (f == null) 
	
				return null;
	
			return f.getAbsolutePath();

		}

		else 

			return super.getParent();

	}

	/**
	 * <p>Returns the file object for abstract pathname of this abstract pathname's parent, or null if this pathname does not name a parent directory.</p>
	 *
	 * @return The file object of the abstract pathname of this abstract pathname's parent or null if this pathname does not name a parent directory
	 *
	 */

	public java.io.File getParentFile() {

		String t = path.replaceFirst(System.getProperty("java.io.tmpdir")+"/", "");

		if (t.equals("."))

			return null;

		if (getLocationInfo()) {

			String s = getAbsolutePath();
			s = trim(s);
		
			int index = s.lastIndexOf(separator);
		
			//this means that a parent exists in the path
			//removing the child from the path		
			if (index != -1) {
				
				s = s.substring(0, index);
	
			}
	
			//no separators found in the path, ergo no parent, returning null
			else {
				
				return null;
	
			}
	
			//adding a separator
			if (!s.endsWith(separator))
		
				s += separator;
	
			//getting all the elements in the path
			String[] elements = vectorize(s);
		
			//checks if any parents present, if not return null
			if (elements.length > 0) {
		
				String temp = "";
		
				for (int i = 0; i < (elements.length); i++) {
		
					temp += elements[i]+separator;
						
				}
	
				//return the new file with the path constructed from the above for-loop
				return new File(temp);
		
			}

			return null;

		}

		else

			return super.getParentFile();

		
	}

	/**
	 * <p>Helper method that removes the separator (normally "/") if the input string starts with it</p>
	 *
	 * @param s - The string to be checked
	 * @return The parsed string, separator removed if the input string started with it
	 *
	 */

	private String trimFirst(String s) {
	
		if (s.startsWith(separator)) 

			return s.substring(1);

		return s;
		
	}

	/**
	 * <p>Helper method that removes separators (normally "/"'s) from the end and the beginning of the 
	 * input string, if they exist</p>
	 *
	 * @param s - The input string to be checked
	 * @return The parsed string
	 *
	 */

	private String trim(String s) {

		if (s.startsWith(separator)) {
 
		        s = s.substring(1);

		}
        
                if (s.endsWith(separator)) {
   
                        s = s.substring(0, s.length()-1);

		}

		return s;

	}

	/**
	 * <p>Returns the canonical form of this abstract pathname. Equivalent to new File(this.getCanonicalPath()()).</p>
	 *
	 * @return The file created from the canonical pathname string denoting the same file or directory as this abstract pathname
	 *
	 */

	public java.io.File getCanonicalFile() throws java.io.IOException {

		if (getLocationInfo()) {

			String s = getCanonicalPath();
			s = s.replaceFirst(System.getProperty("java.io.tmpdir")+"/", "");
		
			return new File(s);

		}
		
		else

			return super.getCanonicalFile();

	}

	/**
	 * <p>Currently not implemented</p>
	 *
	 * @return false
	 *
	 */

	public boolean createNewFile() {

		return false;

	}	

	/**
	 * <p>Checks if the specified input path exists at DFS</p>
	 *
	 * @param path - The path to be checked for existance
	 * @return true if the path is a valid (existing) location at DFS, false otherwise
	 *
	 */

	private boolean existsAtDFS(String path) {

		if (getLocationInfo()) {
                        
                        if (setDFSConnection("https://dfs.cern.ch/"+location+separator+path))
                                
                                try {
                                        
                                        wdr = new WebdavResource(hrl);
                 
                                        return true;
         
                                }
         
                                catch (HttpException httpe) {
        
                                        return false;
         
                                }
         
                                catch (IOException ioe) {

                                        return false;
 
                                }
                
                }

		return false;

	}

	/**
	 * <p>Tests whether the file or directory denoted by this abstract pathname exists at DFS. If no DFS location information is available
	 * it calls the suprclass method</p>
	 *
	 * @return true if the file/directory exists at DFS if DFS location information is available and correct, the return value of the superclass 
	 *		method if not
	 *
	 */

	public boolean exists() {

		if (getLocationInfo()) {

			String temp = path.replaceFirst(System.getProperty("java.io.tmpdir")+"/", "");

			if (temp.equals("."))

				return true;				

			if (setDFSConnection("https://dfs.cern.ch/"+location+separator+getAbsolutePath()))

				try {

					wdr = new WebdavResource(hrl);

					return true;

				}

				catch (HttpException httpe) {

					return false;

				}

				catch (IOException ioe) {

					return false;

				}

		}

		else {

			return super.exists();

		}

		return false;
		
	}

	/**
	 * <p>Returns the absolute form of this abstract pathname. Equivalent to new File(this.getAbsolutePath()())</p>
	 *
	 * @return The File object created from the absolute abstract pathname denoting the same file or directory as this abstract pathname
	 *
	 */
	
	public java.io.File getAbsoluteFile() {

		if (getLocationInfo())

			return new File(getAbsolutePath());

		else 

			return super.getAbsoluteFile();	
	
	}
	
	/**
	 * <p>Converts this abstract pathname into a pathname string. The resulting string uses the default name-separator character to separate the names in the name sequence.</p>
	 *
	 * @return The string form of this abstract pathname
	 *
	 */

	public String getPath() {

		if (getLocationInfo())
		
			return getAbsolutePath();

		else

			return super.getPath();

	}

	/**
	 * <p>Tests whether the file denoted by this abstract pathname is a directory. If information about connection to DFS
	 * is available it checks whether this pathname is a directory at DFS, otherwise the superclass method is returned</p>
	 *
	 * @return true if and only if the file denoted by this abstract pathname exists and is a directory; false otherwise
	 *
	 */

	public boolean isDirectory() {

		if (getLocationInfo()) {    

			String t = path.replaceFirst(System.getProperty("java.io.tmpdir")+"/", "");

			if (t.equals("."))

				return true;

                        if (setDFSConnection("https://dfs.cern.ch/"+location+"/"+getAbsolutePath())) {

				try {

					wdr = new WebdavResource(hrl);

					if (wdr != null) {

						return wdr.isCollection();

					}

					else {

						return false;

					}

				}

				catch (HttpException httpe) {
				
					if (getAbsolutePath().endsWith(separator)) 

						return true;

					return false;

				}

				catch (IOException ioe) {
				
					return false;

				}

			}
		                                
                }

		return super.isDirectory();

	}

	/**
	 * <p>Tests whether the file denoted by this abstract pathname is a normal file. A file is normal if it is not a directory and, 
	 * in addition, satisfies other system-dependent criteria. Any non-directory file created by a Java application is 
	 * guaranteed to be a normal file.</p>
	 *
	 * @return if DFS connection is available: true if and only if the file denoted by this abstract pathname exists and is a normal file; false otherwise.
	 * 		if no connection available: the superclass method return value
	 *
	 */

	public boolean isFile() {

		if (!isDirectory()) 

			return true;

		return false;

	}

	/**
	 * <p>Returns an array of strings naming the files and directories in the directory denoted by this abstract pathname. </p>
	 *
	 * <p>If this abstract pathname does not denote a directory, then this method returns null. Otherwise an array of strings 
	 * is returned, one for each file or directory in the directory. Names denoting the directory itself and the 
	 * directory's parent directory are not included in the result. Each string is a file name rather than a complete path. </p>
	 *
	 * <p>There is no guarantee that the name strings in the resulting array will appear in any specific order; 
	 * they are not, in particular, guaranteed to appear in alphabetical order.</p>
	 *
	 * @return An array of strings naming the files and directories in the directory denoted by this abstract pathname. 
	 *	The array will be empty if the directory is empty. Returns null if this abstract pathname does not denote 
	 *	a directory, or if an I/O error occurs
	 *
	 */

	public String[] list() {

		if (isDirectory()) {

			if (getLocationInfo()) {

				String t = path.replaceFirst(System.getProperty("java.io.tmpdir")+"/", "");

				if (t.equals(".")) {

					if (setDFSConnection("https://dfs.cern.ch/"+location+"/")) {

						try {

							wdr = new WebdavResource(hrl);

							//getting the child WebdavResources from this WebdavResource	
							WebdavResource[] temp = wdr.listWebdavResources();
							String[] returnList = new String[temp.length];
	
							for (int i = 0; i < temp.length; i++) { 
	
								WebdavResource wtemp = temp[i];
	
								String s = (wtemp.getPath()).replaceFirst(wdr.getPath(), "");
	
								if(wtemp.isCollection()) 
	
									returnList[i] = s+"/";
	
								else 
	
									returnList[i] = s;
	
							}
		
							return returnList;	
		
						}

						//means no WebdavResource existed
						catch (HttpException httpe) {}
			
						catch (IOException ioe) {}

					}				

				}
				
				if (setDFSConnection("https://dfs.cern.ch/"+location+"/"+getAbsolutePath())) {

					try {

						wdr = new WebdavResource(hrl);

						//getting the child WebdavResources from this WebdavResource	
						WebdavResource[] temp = wdr.listWebdavResources();
						String[] returnList = new String[temp.length];
	
						for (int i = 0; i < temp.length; i++) { 
	
							WebdavResource wtemp = temp[i];

							String s = (wtemp.getPath()).replaceFirst(wdr.getPath(), "");

							if(wtemp.isCollection()) 

								returnList[i] = s+"/";

							else 

								returnList[i] = s;

						}
	
						return returnList;	
	
					}
	
					//means no WebdavResource existed
					catch (HttpException httpe) {}
			
					catch (IOException ioe) {}
	
				}

			}

			return super.list();

		}

		return null;

	}

	/**
	 * <p>Returns an array of strings naming the files and directories in the directory denoted by this abstract pathname 
	 * that satisfy the specified filter. The behavior of this method is the same as that of the list() method, 
	 * except that the strings in the returned array must satisfy the filter. If the given filter is null then all names are accepted. 
	 * Otherwise, a name satisfies the filter if and only if the value true results when the FilenameFilter.accept(java.io.File, java.lang.String) 
	 * method of the filter is invoked on this abstract pathname and the name of a file or directory in the directory that it denotes.</p>
	 *
	 * @param filter - A filename filter
	 * @return An array of strings naming the files and directories in the directory denoted by this abstract pathname that were accepted 
	 *		by the given filter. The array will be empty if the directory is empty or if no names were accepted by the filter. 
	 *		Returns null if this abstract pathname does not denote a directory, or if an I/O error occurs.
	 *
	 */

	public String[] list(FilenameFilter filter) {

		if (filter == null)

			return list();

		 if (isDirectory()) {

			if (getLocationInfo()) {                        
         
	                        if (setDFSConnection("https://dfs.cern.ch/"+location+"/"+getAbsolutePath())) {
        	                 
                	                try {
                 	
                        	                wdr = new WebdavResource(hrl);
                	
                        	                ArrayList<String> list = new ArrayList<String>();
	
						String[] temp = list();
	
						//adding the files/directories that gets accepted of the filter to the ArrayList
						for (int i = 0; i < temp.length; i++) {
	
							if (filter.accept(this, temp[i]))
		
								list.add(temp[i]);
	
						}
	
						String[] returnList = new String[list.size()];
	
						//creating the return list by transforming the ArrayList
						for (int i = 0; i < list.size(); i++) {
	
							returnList[i] = (String) list.get(i);
	
						}
											                 
	
        	                                return returnList;
                	                 
                        	        }
                	
                        	        catch (HttpException httpe) {
	
						return null;
	
        	                        }
	
        	                        catch (IOException ioe) {
	
						return null;
				
                        	        }
         
                        	}

			}

			return super.list(filter);

                }
                
                return null;

	}

	/**
	 * <p>Returns an array of abstract pathnames denoting the files in the directory denoted by this abstract pathname.</p> 
	 *
	 * <p>If this abstract pathname does not denote a directory, then this method returns null. Otherwise an array of 
	 * File objects is returned, one for each file or directory in the directory. Pathnames denoting the directory 
	 * itself and the directory's parent directory are not included in the result. Each resulting abstract pathname 
	 * is constructed from this abstract pathname using the File(File, String) constructor. Therefore if this 
	 * pathname is absolute then each resulting pathname is absolute; if this pathname is relative then each 
	 * resulting pathname will be relative to the same directory.</p> 
	 *
	 * <p>There is no guarantee that the name strings in the resulting array will appear in any specific order; 
	 * they are not, in particular, guaranteed to appear in alphabetical order.</p>
	 *
	 * @return An array of abstract pathnames denoting the files and directories in the directory denoted by 
	 *		this abstract pathname. The array will be empty if the directory is empty. Returns null if 
	 *		this abstract pathname does not denote a directory, or if an I/O error occurs.
	 *
	 */

	public File[] listFiles() {

		if (!isDirectory()) 

			return null;

		String[] files = list();

		File[] returnFiles = new File[files.length];

		for (int i = 0; i < files.length; i++) {

			returnFiles[i] = new File(files[i]);

		}

		return returnFiles;
			

	}

	/**              
         * <p>Returns an array of abstract pathnames denoting the files and directories in the directory denoted by 
	 * this abstract pathname that satisfy the specified filter. The behavior of this method is the same as that of the 
	 * listFiles() method, except that the pathnames in the returned array must satisfy the filter. If the given filter is 
	 * null then all pathnames are accepted. Otherwise, a pathname satisfies the filter if and only if the 
	 * value true results when the FileFilter.accept(java.io.File, java.lang.String) method of the filter is invoked 
	 * on this abstract pathname and the name of a file or directory in the directory that it denotes.</p>
	 *
	 * @param filter - A filename filter
         * @return An array of abstract pathnames denoting the files and directories in the directory denoted by
         *              this abstract pathname. The array will be empty if the directory is empty. Returns null if
         *              this abstract pathname does not denote a directory, or if an I/O error occurs.
         *
         */

	public File[] listFiles(FileFilter filter) {

		if (filter == null)
        	 
              		return listFiles();
         	
                File[] files = listFiles(filter);
         	
              	return files;

	}

	/**
         * <p>Returns an array of abstract pathnames denoting the files and directories in the directory denoted by 
         * this abstract pathname that satisfy the specified filter. The behavior of this method is the same as that of the
         * listFiles() method, except that the pathnames in the returned array must satisfy the filter. If the given filter is 
         * null then all pathnames are accepted. Otherwise, a pathname satisfies the filter if and only if the
         * value true results when the FileNameFilter.accept(java.io.File, java.lang.String) method of the filter is invoked 
         * on this abstract pathname and the name of a file or directory in the directory that it denotes.</p>
         *
         * @param filter - A filename filter 
         * @return An array of abstract pathnames denoting the files and directories in the directory denoted by
         *              this abstract pathname. The array will be empty if the directory is empty. Returns null if
         *              this abstract pathname does not denote a directory, or if an I/O error occurs.
         * 
         */

	public File[] listFiles(FilenameFilter filter) {

		if (filter == null)

			return listFiles();

		String[] files = list(filter);
                                 
                File[] returnFiles = new File[files.length];
                                
                for (int i = 0; i < files.length; i++) {
                                        
                        returnFiles[i] = new File(files[i]);
                                 
                }
                         
                return returnFiles;

	}

	/**
	 * <p>It the file in question is a directory, this method creates the directory at DFS if DFS location information is available
	 * and valid. If parent directories does not exist, this method will return false (use mkdirs() instead to simultaneously 
	 * create parent directories)</p>
	 *
	 * <p>If no valid DFS location information is available, the superclass method is called and the directory can be created locally</p>
	 *
	 * @return true if and only if the directory was created; false otherwise
	 *
	 */

	public boolean mkdir() {

		String temp = path.replaceFirst(System.getProperty("java.io.tmpdir")+"/", "");

		if (temp.equals("."))

			return false;

		if (!isDirectory()) {

			return false;

		}

		String s = getAbsolutePath();
		
		if (s.startsWith(separator))

			s.substring(1);

		if ((s.split(separator)).length > 1)  {

			return false;

		}

		if (!s.endsWith(separator))

			s += separator;

		java.io.File f = new java.io.File(System.getProperty("java.io.tmpdir")+"/"+s);

		//checking if the directory exists both at DFS and locally
		if (exists() && super.exists()) {

			return true; 		

		}

		//checking if it exists locally
		else if (super.exists()) {

			if (getLocationInfo()) {

				if (setDFSConnection("https://dfs.cern.ch/"+location)) {
                                        
                                        try {   
                                                                                                                     
                                                wdr = new WebdavResource(hrl);

						if (wdr.mkcolMethod(wdr.getPath()+separator+s))
                 
                                                        return true;

					}

					catch (HttpException httpe) {
			              
                                                return false;
                 
                                        }
                                 
                                        catch (IOException ioe) {				             

                                                return false;
                 
                                        }

				}

			}	

		}

		else if (f.mkdir()) {

			if (getLocationInfo()) {

				//checking first if the directory exists at DFS, return true if it does
				if (exists()) {

					return true;

				}

				//if it doesnt exist, try to create it and return the result				
				if (setDFSConnection("https://dfs.cern.ch/"+location)) {

					try {
						
						wdr = new WebdavResource(hrl);
			
						if (wdr.mkcolMethod(wdr.getPath()+separator+s))

							return true;
						
					}

					catch (HttpException httpe) {

						return false;

					}

					catch (IOException ioe) {

						return false;

					}

				}

			} 

		}

		else {

			return false;

		}
	
		return false;		

	}

	/**
	 * <p>This method creates the directory that is denoted by this abstract pathname both locally and at DFS if
	 * valid DFS location information is available. If not, it uses the superclass method to create the directory 
	 * and any non-existent parent directories</p>
	 *
	 * @return true if and only if the directory was created, along with all necessary parent directories; false otherwise
	 *
	 */

	public boolean mkdirs() {

		String temp = path.replaceFirst(System.getProperty("java.io.tmpdir")+"/", "");

		if (temp.equals("."))

			return false;

		if (!isDirectory()) {

			return false;

		}

		String s = getAbsolutePath();
		s = trim(s);

		String[] directories = vectorize(s);
		String path = "";
	
		for (int i = 0; i < directories.length; i++) {
			
			path += directories[i]+separator;

			//creating the dummy file to create the directory locally if it does not exist
			java.io.File f = new java.io.File(System.getProperty("java.io.tmpdir")+"/"+path);

			if (f.exists()) {

				if (existsAtDFS(path)) {

					continue;

				}				

				else if (createDirectory(path)) {

					continue;

				}

				else {

					return false;

				}

			}	

			if (!f.mkdirs()) {

				return false;

			}

			else {

				if (existsAtDFS(path))

					continue;

				else if (createDirectory(path)) {
                                        
                                	continue;
                
                                }
                        
                                else {
                        
                                        return false;
                                 
                                }

			}
			              	
                         
		}

		return true;

	}	

	/** 
	 * <p>This method creates the directory specified by the input string at DFS, if it already doesn't exist</p>
	 *
	 * @param s - The path to the directory that should be created
	 * @return true if the directory was created or already existed, false otherwise
	 *
	 */

	private boolean createDirectory(String s) {

		if (getLocationInfo()) {
                                        
                	if (setDFSConnection("https://dfs.cern.ch/"+location)) {                              
	                                                   
                                try {   
                                                        
                                	wdr = new WebdavResource(hrl);                                                                                        
                
                                        return wdr.mkcolMethod(wdr.getPath()+separator+s);
                                                 
                                }
                                                
                                catch (HttpException httpe) {
                                                        
        	                        return false;
                                                 
                                }
                                                
                                catch (IOException ioe) {

					return false;
                                                 
                                }
         
                       	}
         
        	}

		return false;

	}

	/**
	 * <p>Not implemented</p>
	 *
	 * @return false
	 *
	 */

	public boolean renameTo(java.io.File dest) {

		//not implemented
		return false;

	}

	/**
	 * <p>Returns the name of the file or directory denoted by this abstract pathname. This is just the last name 
	 * in the pathname's name sequence. If the pathname's name sequence is empty, then the empty string is returned.</p>
	 *
	 * @return The name of the file or directory denoted by this abstract pathname, or the empty string if this pathname's name sequence is empty
	 *
	 */

	public String getName() {

		String temp = path.replaceFirst(System.getProperty("java.io.tmpdir")+"/", "");
		
		if (temp.equals("."))

			return temp;

		if (isDirectory()) {

			String s = trim(getAbsolutePath());
			int index = s.lastIndexOf(separator);

			if (index != -1) {

				s = s.substring(index);
				s += separator;
	
				return trimFirst(s);

			}

			s += separator;

			return trimFirst(s);

		}

		else {

			return super.getName();

		}

	}

	/**
	 * <p>Not implemented</p>
	 *
	 * @return null;
	 *
	 */

	public static File[] listRoots() {

		return null;

		

	}

	/**
	 * <p>Not implemented</p>
	 * 
	 * @return null
	 *
	 */

	public static java.io.File createTempFile(String prefix, String suffix, java.io.File directory) {

		return null;

	}

	/**
	 * <p>Not implemented</p>
	 *
	 * @return null
	 *
	 */

	public static java.io.File createTempFile(String prefix, String suffix) {

		return null;

	}

	/** 
	 * <p>Tests whether this abstract pathname is absolute. If a connection to DFS is valid, it will never be. Otherwise, the
	 * superclass method is used</p>
	 * 
	 * @return false if a valid DFS connection is available, the superclass method return value if not
	 *
	 */

	public boolean isAbsolute() {

		if (getLocationInfo()) 

			return false;

		return super.isAbsolute();

	}

}
