package j2ee.io;

import java.util.MissingResourceException;
import java.util.ResourceBundle;

import org.apache.commons.httpclient.URIException;
import org.apache.commons.httpclient.HttpsURL;
import org.apache.commons.httpclient.HttpException;

import java.io.IOException;

import org.apache.webdav.lib.WebdavResource;

/**
 * <p>Extention of the java.io.FileWriter that enables users of the J2EEPS to write their files
 * to their user area at the DFS filesystem at CERN.</p>
 *
 * <p>By instantiating a FileWriter with a j2ee.io.File, writing to the file and eventually calling
 * the flush method of this class, the file will be saved back to DFS if there is valid location 
 * information available as described for the j2ee.io.File class</p>
 *
 * <p>It is worth noting that this class will overwrite already existing files at DFS if the normal
 * constructor is used, so be careful if you don't want to delete your data unintentially</p>
 *
 * @author Ola Holmesad
 *
 */

public class FileWriter extends java.io.FileWriter {

	private String location = "";
	private String username = "";
	private String password = "";

	private java.io.File file = null;

	private HttpsURL hrl = null; 

	/**
 	 * <p>Standard constructor that creates the FileWriter from the input file</p>
  	 *
 	 * @param file - The file to be written to
 	 *
	 */

	public FileWriter(File file) throws java.io.IOException {

		super(System.getProperty("java.io.tmpdir")+"/"+file.getAbsolutePath());

		this.file = file;

	}

	/**
	 * <p>Constructs a FileWriter object given a File object. If the second argument is true, 
	 * then bytes will be written to the end of the file rather than the beginning.</p>
	 *
	 * @param file - The file to be written to
	 * @param append - if true, the bytes will be written to the end of the file rather than the beginning
	 *
	 */

	public FileWriter(File file, boolean append) throws java.io.IOException {

		super(System.getProperty("java.io.tmpdir")+"/"+file.getAbsolutePath(), append);

		this.file = file;

	}

	/**
	 * <p>Constructs a FileWriter object given a file name.</p>
	 *
	 * param fileName - The filename to write to
	 *
	 */

	public FileWriter(String fileName) throws java.io.IOException {

		super(System.getProperty("java.io.tmpdir")+"/"+fileName);

		file = new File(fileName);

	}

	/**
	 * <p>Constructs a FileWriter object given a file name with a boolean indicating whether or not to append the data written.</p>
	 *
	 * @param fileName - The filename to write to
	 * @param append - f true, then data will be written to the end of the file rather than the beginning.
	 *
	 */

	public FileWriter(String fileName, boolean append) throws java.io.IOException {

		super(System.getProperty("java.io.tmpdir")+"/"+fileName, append);

		file = new File(fileName);

	}

	/**
	 * <p>Method that actually saves the file to the location at DFS. It is used by the flush method</p>
	 *
	 */

	private void save() {

		if (setDFSConnection("https://dfs.cern.ch/"+location+"/"+file.getAbsolutePath())) {
       
			try {

				WebdavResource wdr = new WebdavResource(hrl);

                        	//if we get here, it means that file does exist at DFS

                        	try {
                                
					setDFSConnection("https://dfs.cern.ch/"+location); 
	
        	                        wdr = new WebdavResource(hrl);
	
					wdr.putMethod(wdr.getPath()+"/"+file.getAbsolutePath(), new java.io.File(System.getProperty("java.io.tmpdir")+"/"+file.getAbsolutePath()));                        
         	
                	        }
                        	 
	                        catch (HttpException httpe) {
        	                                                                
					return;
                                	 
                        	}
                         
	                        catch (IOException ioe) {
        	                         
					return;
                                 
                		}

			}

			catch (HttpException httpe) {

				//if we are here, this means that the file does not exist
				if (setDFSConnection("https://dfs.cern.ch/"+location)) {
                                 
                                	try {   
         
                                        	java.io.File parent = file.getParentFile();
                        	
						//if no parent we don't have to make any directories locally to make this work
                                	        if (parent == null) {
                         	
                                	                WebdavResource wdr = new WebdavResource(hrl);
                                	                wdr.putMethod(wdr.getPath()+"/"+file.getAbsolutePath(), new java.io.File(System.getProperty("java.io.tmpdir")+"/"+file.getAbsolutePath()));
                                	
                                        	        return;
                 	
                        	                }
         	
						//if only one parent before temp directory, we should create this one directory locally
                	                        else if (parent.mkdir()) {
                        	
                                	                WebdavResource wdr = new WebdavResource(hrl);
                                	                wdr.putMethod(wdr.getPath()+"/"+file.getAbsolutePath(), new java.io.File(System.getProperty("java.io.tmpdir")+"/"+file.getAbsolutePath()));
                        	
                                	                return;
                        	
                                	        }
                                	
						//if several parent directories before temp directory we have to create them all
                                        	else if (parent.mkdirs()) {
                                        	
                                                	WebdavResource wdr = new WebdavResource(hrl);
							wdr.putMethod(wdr.getPath()+"/"+file.getAbsolutePath(), new java.io.File(System.getProperty("java.io.tmpdir")+"/"+ file.getAbsolutePath()));
	
							return;
                	                                
                        	                }
                                	                
                                        	else
                                         	
                                                	return;
                                        	
                                	}	
                                        		        
	                                catch (HttpException httpe2) {
        	                                        
                	                        return;
                        	 
                                	}
                                         
                                	catch (IOException ioe) {
                         	
                                	        return;
                         	
                                	}

				}

			}

			catch (IOException ioe) {}

			return;			         
               
             	}

	}

	/**
	 * <p>This method flushes the data written to the file first locally, and then, if DFS location information is
	 * available and valid, it saves the data to DFS</p>
	 *
	 */

	public void flush() throws IOException {

		super.flush();

		if (getLocationInfo()) {
	
			save();
	
			return;
		}	

	}

	/**
         * <p>This method sets the DFS connection according to the specified input. It does not check if this
         * is a valid location, that has to be done after by trying to create the WebdavResource connection</p>
         *
         * @param location - the DFS location
         * @return true if the connection is okay, false otherwise
         *
         */

	public boolean setDFSConnection(String location) {
                                
                try {
                
                        hrl = new HttpsURL(location);
                        hrl.setUserinfo(username, password);
         
                        return true;    

                }

                catch (HttpException httpe) {

                        return false;
                        
                }
                        
                catch (URIException urie) {
                 
                        return false;
         
                }

        }
	
	/**
         * <p>Helper method that ensures that the input does not start or end with a "/". Is used
         * to ensure conformity when treating location strings in the class</p>
         *
         * @param s - The string to be checked for starting or ending "/"'s
         * @return - The parsed string
         *
         */

	private String parseLocation(String s) {
                
                if (s.startsWith(File.separator)) {

                        s = s.substring(1);

                }

                if (s.endsWith(File.separator)) {
         
                        int length = s.length();
                        s = s.substring(0, s.length()-1);
          
                }
         
                return s;
           
        }

	/**
         * <p>Checks if it can find a dfs.properties file in the user's directory. </p>
         *
         * @return true if a connection is possible to DFS, false otherwise
         *
         */

	private boolean getLocationInfo() {

                try {
                        
                        ResourceBundle bundle = ResourceBundle.getBundle("dfs");

                        location = parseLocation(bundle.getString("dfs.location"));
                        username = bundle.getString("dfs.username");
                        password = bundle.getString("dfs.password");

			return true;

                }

                catch(MissingResourceException mre) {

                        return false;

                }

        }	

}
